/*
 * $Id: fancybufferchannel.h 909 2011-05-27 21:20:49Z fancylibs $
 *
 * Author: Christoph Lechleitner/IBCL <lech@ibcl.at>
 *
 * Header of BufferChannel
 * Channels reading/writing from/to in-RAM buffers with base64 encoding/decoding on write operations
 *
 */

#ifndef FANCY_BUFFERCHANNEL_H_
#define FANCY_BUFFERCHANNEL_H_

#include <fancychannel.h>
#include <fancyobject.h>

namespace Fancy {

template<class T> class ArrayObject;

/*!
 * A Channel that operates on an in-memory buffer, i.e. a uint8_t*,
 * and allows direct access to this buffer.
 */
class FANCYBASE_API BufferChannel : public virtual Channel {

public:
    typedef ObjPtr<BufferChannel> ptr_t;

    /** returns a BufferChannel instance operating on a buffer with bufsize bytes.
     * Initially data can only be written (via Channel methods), unless set with setData()/appendData().
     */
    static ptr_t newInstance(int bufsize);

    /** copies up to count bytes from data into the buffer and sets writePos to the end of the written data, sets readPos to 0.
     * checks bufsize quietly, i.e. does not crash nor throw an exception, just swallows excessive data.
     * setData(0,0) safely resets the buffer.
     */
    virtual void setData(const void *data, int count) = 0;

    /** copies up to count bytes from data into the buffer and sets writePos to the end of the written data, sets readPos to 0.
     * checks bufsize quietly, i.e. does not crash nor throw an exception, just swallows excessive data.
     * setData(0,0) safely resets the buffer.
     */
    virtual void setData(const ObjPtr<ArrayObject<int8_t> > &data, int count = -1) = 0;

    /** copies count bytes of the buffer to data which must have count bytes reserved */
    virtual void getData(void *data, int count) const = 0;

    /** returns a pointer to the data buffer. very dangerous, but useful for i.e. cheap logging. */
    virtual uint8_t* getDataPtr() const = 0;

    /** copies up to count bytes from data to the buffer[writePos] and sets writePos to the end of the written data, does not touch readPos.
     * checks bufsize quietly, i.e. does not crash nor throw an exception, just swallows excessive data.
     */
    virtual void appendData(const void *data, int count) = 0;

    /** copies up to count bytes from data to the buffer[writePos] and sets writePos to the end of the written data, does not touch readPos.
     * checks bufsize quietly, i.e. does not crash nor throw an exception, just swallows excessive data.
     */
    virtual void appendData(const ObjPtr<ArrayObject<int8_t> > &data, int count = -1) = 0;

    /** sets buffer[0-count] to fillByte, sets writePos to count, readPos to 0.
     * checks bufsize quietly, i.e. does not crash nor throw an exception, just ignores excessive fill wishes.
     */
    virtual void fillBuffer(int count, uint8_t fillByte=0) = 0;

    /** returns the current read position */
    virtual int getReadPos() const = 0;

    /** returns the current write position */
    virtual int getWritePos() const = 0;


    // from Channel
    virtual int read (void *buf, int count) = 0;
    virtual int write (const void *buf, int count) = 0;
    virtual void flush() = 0;
    virtual void close() = 0;
    virtual bool isOpen() const = 0;
    virtual void shutdown(ShutdownOption o) = 0;


};


}

#endif /* FANCY_BUFFERCHANNEL_H_ */
