/*
 * fancyserializable.h
 *
 *  Created on: 07.05.2009
 *      Author: wglas
 */

#ifndef FANCYSERIALIZABLEFACTORY_H_
#define FANCYSERIALIZABLEFACTORY_H_

#include <fancyserializable.h>
#include <fancylist.h>

namespace Fancy {

/*!
 * The interface for a factory of serializable objects.
 */
class FANCYBASE_API SerializableFactory: public virtual Fancy::Object {
public:
    typedef Fancy::ObjPtr<SerializableFactory> ptr_t;

    /**
     * Return a new default implementation. The returned instance is aware of all
     * Serializable classes in the library @c fancybase.
     */
    static ptr_t newInstance();

    /**
     * Return a new default implementation using the given delegate for resolving unknown
     * serial IDs.
     *
     * The returned instance is not aware of any Serializable class in the library @c fancybase.
     */
    static ptr_t newInstance(const ptr_t &delegate);

    SerializableFactory() {}

    /*!
     *
     * @param id The object ID as returned by \c Serializable::getSerialID().
     * @param serialVersionUID The object version ID as returned by
     *                         \c Serializable::getSerialVersionUID().
     * @return a new object of the given type id.
     */
    virtual ObjPtr<Serializable> newSerializable(const std::string &id, int64_t serialVersionUID) const = 0;

    /**
     * @param id The object ID as returned by @c Serializable::getSerialID().
     * @param serialVersionUID The object version ID as returned by
     *                         @c Serializable::getSerialVersionUID().
     * @param sz The size of the returned array.
     * @return an instance of @c Fancy::ArrayObjectList<T> where T is the of the given type id.
     */
    virtual ObjPtr<List> newSerializableArray(const std::string &id, int64_t serialVersionUID, size_t sz)  const = 0;

    /**
     * Register a serializable class using a constructor function.
     */
    virtual void registerClass(const char *id, int64_t serialVersionUID, newSerializableInstance_t ctor) = 0;

    /**
     * Register a serializable class using a constructor function and a set of compatibility serial version UIDs.
     */
    virtual void registerClass(const char *id, int64_t serialVersionUID,
                               getOldSerialVersionUIDs_s_t getOSVUIDs,
                               newSerializableInstance_t ctor) = 0;

    /**
     * Register a serializable class array using a constructor function.
     */
    virtual void registerClassArray(const char *id, int64_t serialVersionUID,
                                    newArrayObjectListInstance_t newArrayObjectList) = 0;

    /**
     * Register a serializable class array using a constructor function and a set of compatibility serial version UIDs.
     */
    virtual void registerClassArray(const char *id, int64_t serialVersionUID,
                                    getOldSerialVersionUIDs_s_t getOSVUIDs,
                                    newArrayObjectListInstance_t newArrayObjectList) = 0;

    /**
     * Register a serializable class by its static @c getSerialID_s, @c getSerialVersionUID_s
     * and @c newSerializableInstance functions.
     */
    template <class T> void registerSerializable() {
        this->registerClass(T::getSerialID_s(),T::getSerialVersionUID_s(),T::getOldSerialVersionUIDs_s,T::newSerializableInstance);
    }

    /**
     * Register a serializable class as array component by its static @c getSerialID_s and
     * @c getSerialVersionUID_s functions as well as @c ObjectArray<T>::newListInstance.
     */
    template <class T> void registerSerializableArray() {
        this->registerClassArray(T::getSerialID_s(),T::getSerialVersionUID_s(),
                                 T::getOldSerialVersionUIDs_s,ObjectArray<T>::newListInstance);
    }
    /**
     * Register a serializable class for normal fields and array components by calling
     * @c registerSerializable() as well as @c registerSerializableArray().
     */
    template <class T> void registerSerializableAll() {
        this->registerSerializable<T>();
        this->registerSerializableArray<T>();
    }

protected:
    ~SerializableFactory();
};

}

#endif /* FANCYSERIALIZABLE_H_ */
