/*
 * fancystring.h
 *
 *  Created on: 17.05.2009
 *      Author: wglas
 */

#ifndef FANCYSTRING_H_
#define FANCYSTRING_H_

#include <fancyserializable.h>
#include <stdarg.h>

namespace Fancy {

/**
 * A base class for string implementation. The default implementation
 * is based on @c std::string, but for GUI programs an alternative implementation
 * may be provided based on @c QString.
 */
class FANCYBASE_API String: public virtual Fancy::Serializable {
public:
    typedef ObjPtr<String> ptr_t;

    String() {}

    static ObjPtr<Serializable> newSerializableInstance();

    /**
     * Return a reference to an empty string. This constructor is
     * mainly needed during a deserialization sequence, when
     * @c Serializable::readFields will be called subsequently.
     */
    static ObjPtr<String> newInstance();

    /**
     * Test the given C string and object pointer for equality.
     * This is an optimized version of
     * @verbatim
     * Fancy::fwd_equals(String::newInstance(s),o)
     * @endverbatim
     */
    static bool c_equals(const char *s, const ObjPtr<Object> &o);

    /**
     * Test the given object pointer and C string for equality.
     * This is an optimized version of
     * @verbatim
     * Fancy::fwd_equals(o,String::newInstance(s))
     * @endverbatim
     */
    static bool c_equals(const ObjPtr<Object> &o, const char *s);

    /**
     * Compare the given C string and object pointer.
     * This is an optimized version of
     * @verbatim
     * Fancy::fwd_less(String::newInstance(s),o)
     * @endverbatim
     */
    static bool c_less(const char *s, const ObjPtr<Object> &o);

    /**
     * Compare the given object pointer and C string.
     * This is an optimized version of
     * @verbatim
     * Fancy::fwd_equals(o,String::newInstance(s))
     * @endverbatim
     */
    static bool c_less(const ObjPtr<Object> &o, const char *s);

    /**
     * Return a new string instance representing the given UTF-8 string.
     * If @c s is NULL, a null reference is returned.
     */
    static ObjPtr<String> newInstance(const char *s);

    /**
     * Register the implementation returned by all @c newInstance functions.
     * If not registered here, a default implementation based on @c std::string
     * is registered.
     *
     * This function mainly exists in order to make GUI libraries like Qt play well
     * with fancylibs. The library @c fancyqt4 provides for a more sophisticated
     * @c String implementation, which stores a @c QString and a cached utf-8
     * representation based .
     */
    static void registerImplementation(String *(*newString_1)(),
                                       String *(*newString_2)(const char *),
                                       String *(*newString_3)(const std::string &));

    /**
     * Return a new string instance representing the given UTF-8 string.
     */
    static ObjPtr<String> newInstance(const std::string &s);

    /**
     * Return a new string instance representing the given UTF-8 string.
     * This version always return the internal @c std::string default
     * implementation regardless, which implementation has been registered
     * using @c registerImplementation.
     *
     * This method is of limited interest for normal users. However, it might
     * prove useful in @c String implementations, which internally cache
     * a @c std::string instance.
     */
    static ObjPtr<String> newStdInstance(const std::string &s);

    /**
     * Format a string using a printf-style format string and format
     * arguments.
     */
    #ifdef __GNUG__
      __attribute__((format(printf,1,2)))
#endif
    static ObjPtr<String> newFmtInstance(const char *fmt, ...);

    /**
     * Format a string using a printf-style format string and a stdarg
     * argument list like in @c vsnprintf.
     */
    static ObjPtr<String> newVFmtInstance(const char *fmt, va_list ap);

    /**
     * Format a @c std::string using a printf-style format string and stdarg
     * argument list like in @c vsnprintf.
     */
#ifdef __GNUG__
      __attribute__((format(printf,1,2)))
#endif
    static std::string sprintf(const char *fmt, ...);

    /**
     * Format a @c std::string using a printf-style format string and a
     * arguments.
     */
    static std::string vsprintf(const char *fmt, va_list ap);

    virtual std::string getUtf8String() const = 0;

    virtual size_t getUtf8Length() const = 0;

    static int64_t getSerialVersionUID_s();
    static const char *getSerialID_s();

    /**
     * @return A pseudo serial version UID, which is passed to @c readFields, when
     *         the string contetn should be read in the long format by using
     *         @c DataChannel.readUTFLong().
     */
    static int64_t getLongSerialVersionUID_s();

    int64_t getSerialVersionUID() const;
    const char *getSerialID() const;

protected:
    virtual ~String();
};

}

#endif /* FANCYSTRING_H_ */
